@@ -1,4 +1,9 @@
-use std::{cmp::Ordering, convert::{Infallible, TryFrom, TryInto}, fmt::Display, str::FromStr};
+use std::{
+    cmp::Ordering,
+    convert::{TryFrom, TryInto},
+    fmt::Display,
+    str::FromStr,
+};
 
 use serde::{Deserialize, Serialize};
 
@@ -8,7 +13,8 @@ use serde::{Deserialize, Serialize};
 pub struct IdString([u8; 26]);
 
 impl IdString {
-    fn check(s: &str) -> Result<(), IdStringDeserializeError> {
+    /// Checks whether the string is a valid Id
+    pub fn check(s: &str) -> Result<(), IdStringDeserializeError> {
         if s.len() != 26 {
             return Err(IdStringDeserializeError::IncorrectLength {
                 expected: 26,
@@ -16,26 +22,40 @@ impl IdString {
             });
         }
 
-        match s.find(|c: char| !('A'..='Z').contains(&c) && !('0'..='9').contains(&c)) {
-            Some(pos) => {
-                let c = s.chars().nth(pos).unwrap();
+        if let Some(pos) = s.find(|c: char| !('A'..='Z').contains(&c) && !('0'..='9').contains(&c))
+        {
+            let c = s.chars().nth(pos).unwrap();
 
-                return Err(IdStringDeserializeError::InvalidCharacter { c, pos });
-            }
-
-            None => {}
+            return Err(IdStringDeserializeError::InvalidCharacter { c, pos });
         }
 
         Ok(())
     }
 
+    /// Creates a new [`IdString`] wihtout checking that
+    /// the contents are valid id characters, and the whole string
+    /// is of the right length.
+    /// # Safety
+    /// s should have passed [Self::check]
     pub unsafe fn from_str_unchecked(s: &str) -> Self {
         Self(s.as_bytes().try_into().unwrap())
     }
 
+    /// Creates a new [`IdString`] wihtout checking that
+    /// the contents are valid id characters, and the whole string
+    /// is of the right length.
+    /// # Safety
+    /// s should have passed [Self::check]
     pub unsafe fn from_string_unchecked(s: String) -> Self {
         Self(s.as_bytes().try_into().unwrap())
     }
+
+    /// Gets the time the object identified by this id was created at.
+    pub fn datetime(&self) -> chrono::DateTime<chrono::Utc> {
+        let ulid = self.as_ref().parse::<rusty_ulid::Ulid>().unwrap();
+
+        ulid.datetime()
+    }
 }
 
 /// An error that can occur while parsing an IdString
@@ -95,30 +115,18 @@ impl PartialEq<String> for IdString {
     fn eq(&self, other: &String) -> bool {
         self.as_ref().eq(other)
     }
-
-    fn ne(&self, other: &String) -> bool {
-        self.as_ref().ne(other)
-    }
 }
 
 impl PartialEq<str> for IdString {
     fn eq(&self, other: &str) -> bool {
         self.as_ref().eq(other)
     }
-
-    fn ne(&self, other: &str) -> bool {
-        self.as_ref().ne(other)
-    }
 }
 
 impl<'a> PartialEq<&'a str> for IdString {
     fn eq(&self, other: &&'a str) -> bool {
         self.as_ref().eq(*other)
     }
-
-    fn ne(&self, other: &&'a str) -> bool {
-        self.as_ref().ne(*other)
-    }
 }
 
 impl PartialOrd<String> for IdString {
@@ -139,22 +147,175 @@ impl<'a> PartialOrd<&'a str> for IdString {
     }
 }
 
+/*
+
+Variable length id strings
+
+*/
+
+/// variable length(up to 26 bytes) of numeric or letter characters
+#[derive(Serialize, Deserialize, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[serde(try_from = "String", into = "String")]
+pub struct VarLenIdString([u8; 26], usize);
+
+impl VarLenIdString {
+    /// Checks whether the string is a valid Id
+    pub fn check(s: &str) -> Result<(), VarLenIdStringDeserializeError> {
+        if s.len() > 26 {
+            return Err(VarLenIdStringDeserializeError::IncorrectLength {
+                expected_most: 26,
+                len: s.len(),
+            });
+        }
+
+        if let Some(pos) = s.find(|c: char| {
+            !('A'..='Z').contains(&c) && !('a'..='z').contains(&c) && !('0'..='9').contains(&c)
+        }) {
+            let c = s.chars().nth(pos).unwrap();
+
+            return Err(VarLenIdStringDeserializeError::InvalidCharacter { c, pos });
+        }
+
+        Ok(())
+    }
+
+    /// Creates a new [`IdString`] wihtout checking that
+    /// the contents are valid id characters, and the whole string
+    /// is of the right length.
+    /// # Safety
+    /// s should have passed [Self::check]
+    pub unsafe fn from_str_unchecked(s: &str) -> Self {
+        let len = s.len();
+        let mut buf = [0; 26];
+        buf[..len].copy_from_slice(s.as_bytes());
+
+        Self(buf, len)
+    }
+
+    /// Creates a new [`IdString`] wihtout checking that
+    /// the contents are valid id characters, and the whole string
+    /// is of the right length.
+    /// # Safety
+    /// s should have passed [Self::check]
+    pub unsafe fn from_string_unchecked(s: String) -> Self {
+        Self::from_str_unchecked(&s)
+    }
+}
+
+/// An error that can occur while parsing an IdString
+#[derive(thiserror::Error, Debug)]
+pub enum VarLenIdStringDeserializeError {
+    #[error("invalid character '{c}' at position {pos}")]
+    InvalidCharacter { pos: usize, c: char },
+    #[error("incorrect length: is {len}, expected at most {expected_most}")]
+    IncorrectLength { len: usize, expected_most: usize },
+}
+
+impl FromStr for VarLenIdString {
+    type Err = VarLenIdStringDeserializeError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Self::check(s)?;
+
+        Ok(unsafe { Self::from_str_unchecked(s) })
+    }
+}
+
+impl TryFrom<String> for VarLenIdString {
+    type Error = VarLenIdStringDeserializeError;
+
+    fn try_from(s: String) -> Result<Self, Self::Error> {
+        Self::check(&s)?;
+
+        Ok(unsafe { Self::from_string_unchecked(s) })
+    }
+}
+
+impl AsRef<str> for VarLenIdString {
+    fn as_ref(&self) -> &str {
+        unsafe { std::str::from_utf8_unchecked(&self.0[..self.1]) }
+    }
+}
+
+impl From<VarLenIdString> for String {
+    fn from(id: VarLenIdString) -> Self {
+        id.as_ref().to_string()
+    }
+}
+
+impl std::fmt::Debug for VarLenIdString {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        <str as Display>::fmt(self.as_ref(), f)
+    }
+}
+
+impl Display for VarLenIdString {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        self.as_ref().fmt(f)
+    }
+}
+
+impl PartialEq<String> for VarLenIdString {
+    fn eq(&self, other: &String) -> bool {
+        self.as_ref().eq(other)
+    }
+}
+
+impl PartialEq<str> for VarLenIdString {
+    fn eq(&self, other: &str) -> bool {
+        self.as_ref().eq(other)
+    }
+}
+
+impl<'a> PartialEq<&'a str> for VarLenIdString {
+    fn eq(&self, other: &&'a str) -> bool {
+        self.as_ref().eq(*other)
+    }
+}
+
+impl PartialOrd<String> for VarLenIdString {
+    fn partial_cmp(&self, other: &String) -> Option<Ordering> {
+        Some(self.as_ref().cmp(other))
+    }
+}
+
+impl PartialOrd<str> for VarLenIdString {
+    fn partial_cmp(&self, other: &str) -> Option<Ordering> {
+        Some(self.as_ref().cmp(other))
+    }
+}
+
+impl<'a> PartialOrd<&'a str> for VarLenIdString {
+    fn partial_cmp(&self, other: &&'a str) -> Option<Ordering> {
+        Some(self.as_ref().cmp(*other))
+    }
+}
+
 macro_rules! id_impl {
-    ($name:ident) => {
-        impl From<IdString> for $name {
-            fn from(id: IdString) -> Self {
+    (@dt $name:ident, $base_ty:ty) => {
+        impl $name {
+            pub fn datetime(&self) -> chrono::DateTime<chrono::Utc> {
+                self.0.datetime()
+            }
+        }
+
+        id_impl!($name, $base_ty);
+    };
+    ($name:ident, $base_ty:ty) => {
+        impl From<$base_ty> for $name {
+            fn from(id: $base_ty) -> Self {
                 Self(id)
             }
         }
 
-        impl From<$name> for IdString {
+        impl From<$name> for $base_ty {
             fn from(id: $name) -> Self {
                 id.0
             }
         }
 
         impl FromStr for $name {
-            type Err = <IdString as FromStr>::Err;
+            type Err = <$base_ty as FromStr>::Err;
 
             fn from_str(s: &str) -> Result<Self, Self::Err> {
                 s.parse().map(Self)
@@ -162,10 +323,10 @@ macro_rules! id_impl {
         }
 
         impl TryFrom<String> for $name {
-            type Error = <IdString as TryFrom<String>>::Error;
+            type Error = <$base_ty as TryFrom<String>>::Error;
 
             fn try_from(value: String) -> Result<Self, Self::Error> {
-                IdString::try_from(value).map(Self)
+                <$base_ty>::try_from(value).map(Self)
             }
         }
 
@@ -191,44 +352,32 @@ macro_rules! id_impl {
             fn eq(&self, other: &String) -> bool {
                 self.as_ref().eq(other)
             }
-        
-            fn ne(&self, other: &String) -> bool {
-                self.as_ref().ne(other)
-            }
         }
-        
+
         impl PartialEq<str> for $name {
             fn eq(&self, other: &str) -> bool {
                 self.as_ref().eq(other)
             }
-        
-            fn ne(&self, other: &str) -> bool {
-                self.as_ref().ne(other)
-            }
         }
-        
+
         impl<'a> PartialEq<&'a str> for $name {
             fn eq(&self, other: &&'a str) -> bool {
                 self.as_ref().eq(*other)
             }
-        
-            fn ne(&self, other: &&'a str) -> bool {
-                self.as_ref().ne(*other)
-            }
         }
-        
+
         impl PartialOrd<String> for $name {
             fn partial_cmp(&self, other: &String) -> Option<Ordering> {
                 Some(self.as_ref().cmp(other))
             }
         }
-        
+
         impl PartialOrd<str> for $name {
             fn partial_cmp(&self, other: &str) -> Option<Ordering> {
                 Some(self.as_ref().cmp(other))
             }
         }
-        
+
         impl<'a> PartialOrd<&'a str> for $name {
             fn partial_cmp(&self, other: &&'a str) -> Option<Ordering> {
                 Some(self.as_ref().cmp(*other))
@@ -242,36 +391,56 @@ macro_rules! id_impl {
 #[serde(transparent)]
 pub struct UserId(IdString);
 
-id_impl! {UserId}
-
+id_impl! {@dt UserId, IdString}
 
 /// Id type for channels.
 #[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
 #[serde(transparent)]
 pub struct ChannelId(IdString);
 
-id_impl! {ChannelId}
+id_impl! {@dt ChannelId, IdString}
+
+/// Id type for channels.
+#[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[serde(transparent)]
+pub struct CategoryId(VarLenIdString);
+
+id_impl! {CategoryId, VarLenIdString}
 
 /// Id type for messages.
 #[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
 #[serde(transparent)]
 pub struct MessageId(IdString);
 
-id_impl! {MessageId}
+id_impl! {@dt MessageId, IdString}
 
 /// Id type for servers.
 #[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
 #[serde(transparent)]
 pub struct ServerId(IdString);
 
-id_impl! {ServerId}
+id_impl! {@dt ServerId, IdString}
 
 /// Id type for roles.
 #[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
 #[serde(transparent)]
 pub struct RoleId(IdString);
 
-id_impl! {RoleId}
+id_impl! {@dt RoleId, IdString}
+
+/// Id type for sessions.
+#[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[serde(transparent)]
+pub struct SessionId(IdString);
+
+id_impl! {@dt SessionId, IdString}
+
+/// Id type for invites.
+#[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[serde(transparent)]
+pub struct InviteId(IdString);
+
+id_impl! {@dt InviteId, IdString}
 
 /// Id type for members.
 ///
@@ -281,55 +450,3 @@ pub struct MemberId {
     pub server: ServerId,
     pub user: UserId,
 }
-
-/// Id type for attachments
-///
-/// Attachment ids are returned by `Autumn`.
-// and can be from 1 up to 128 characters
-// right now uploading a file gives a 42
-// char id
-#[derive(Serialize, Deserialize, Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
-#[serde(try_from = "String", into = "String")]
-pub struct AttachmentId([u8; 128], usize); // buffer + string slice length
-
-impl FromStr for AttachmentId {
-    type Err = Infallible;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Self::from(s))
-    }
-}
-
-impl<'a> From<&'a str> for AttachmentId {
-    fn from(s: &'a str) -> Self {
-        let len = s.len();
-        let mut buf = [0; 128];
-        buf[..len].copy_from_slice(s.as_bytes());
-
-        Self(buf, len)
-    }
-}
-
-impl From<String> for AttachmentId {
-    fn from(s: String) -> Self {
-        Self::from(s.as_str())
-    }
-}
-
-impl AsRef<str> for AttachmentId {
-    fn as_ref(&self) -> &str {
-        unsafe { std::str::from_utf8_unchecked(&self.0[..self.1]) }
-    }
-}
-
-impl From<AttachmentId> for String {
-    fn from(id: AttachmentId) -> Self {
-        id.as_ref().to_string()
-    }
-}
-
-impl Display for AttachmentId {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        self.as_ref().fmt(f)
-    }
-}
